#ifndef TRXTARG_H
/** \file
   Various definitions for target materials.
*/
/*

   Copyright  1992-2010 M.Kraemer, GSI, D-64291 Darmstadt

   $Log: trxtarg.h,v $
   Revision 1.5  2009/05/29 12:25:02  kraemer
   rework targets

   Revision 1.4  2008/06/20 14:36:15  kraemer
   x98-libs

   Revision 1.3  2002/11/25 16:37:36  kraemer
   TEA (tri-ethyl-amine) added

   Revision 1.2  2002/01/22 17:31:29  kraemer
   strict for Aix, OS-dependent makefile, bugfix in mdose evaluation

   Revision 1.1  2001/01/09 14:54:42  kraemer
   *** empty log message ***



*/
#define TRXTARG_H

#include "utypes.h"
#include "ulists.h"
#include "ufilemem.h"
#include "m.h"
#include "cmderr.h"
#include "trxproj.h"
                                      /* */
                                      /* global flags */
#define TRXTARG_DEBUG  0x80000000     /* general check flag */
#define TRXTARG_QUIET  0x10000000     /* suppress error messages */
#define TRXTARG_NOMSG  TRXTARG_QUIET
#define TRXTARG_DUPASREF 0x04000000   /* duplicate as a reference copy, i.e. no allocation */

                                      /** \struct STRXXSTAG
                                          \brief Aux. structure to correlate tag string <=> ID.
                                      */ 
struct STRXTARGTAGID { char *pcTag; int iID; };

                                      /** \struct STRXPARENT
                                          \brief  describes parentage 
                                      */
struct STRXPARENT {
   double dFraction;                  /**< parentage coeff. */
   char cAtom[4];                     /**< parent atom name */
   char *pcCfg;                       /**< parent atomic shell name */
};                                          
                                      /** \struct STRXSHELL 
                                          \brief describes an electron shell 
                                      */
struct STRXSHELL {
   char *pcCfg;                       /**< shell config */
   int in, il;                        /**< quantum numbers */ 
   int    ne;                         /**< # of electrons */
   double dEbnd,                      /**< experimental binding energy */
          dEkin;                      /**< mean kinetic energy a la Slater */
   double ds;                         /**< shielding constant */
   int nParent;                       /**< # of parent atoms (0 if none) */
   struct STRXPARENT *psParent;       /**< parent atoms */  
};
                                       /** \struct STRXTARG 
                                          \brief Describes target material 
                                      */
struct STRXTARG {
   unsigned int uiFlag;               /**< holds flag bits */

                                      /* flag bits: */
#define TRXTARG_REFCOPY   0x00000001  /* do not free() */ 
   unsigned int uiCode;               /**< short cut material code number */ 
   unsigned int uiFileVersion;        /**< file version number (on read), yyyymmdd */ 

   size_t usztSpace;                  /**< space occupied by this structure */
   char *pcMaterial;                  /**< material name */
   char *pcComposition;               /**< chemical composition */
   struct STRXZA sZA;                 /**< atom species (if no further constituents) */
   double dN;                         /**< relative contribution or number of atoms */
   double dNSum;                      /**< ??? */
   double dDensity, dMolWeight;       /**< density [g/cm**3], molweight [g] */
   double dnMolecule, dnNuclei;       /**< molecule, nuclei number density */
                                      /* NA / MolWeight */
   int nT;                            /**< # of contributing materials/atoms (from decomposition) */ 
   struct STRXTARG *psT;              /**< [nT] contributing materials/atoms */   
                                      /* molecular shells */  
   int nSh;                           /**< # of shells */
   struct STRXSHELL *psSh;            /**< [nSh] shells */
};
                                      /* commonly used short codes */
enum {
   TRXTARG_NONE=0, 
   TRXTARG_H2O=1, TRXTARG_C, TRXTARG_C2H4, TRXTARG_N2, TRXTARG_O2,
   TRXTARG_PMMA,
   TRXTARG_AL, TRXTARG_NAI, TRXTARG_AGBR, 
   TRXTARG_MYLAR,
   TRXTARG_TEA,
};				      

                                      /* short test whether target material is a mixture */
#define TRXTARGISMIXTURE(t) ( ((t)->nSh <= 0) && ((t)->nT > 0) )

                                      /* target file format version */
/*
#define TRXTARGFILEVERSION 19990513
*/
#define TRXTARGFILEVERSION 20020403


/*

   Functions to handle a single target material
   --------------------------------------------

   TRXTargCalc   ( t, projbase, targname, flag)
                 o Try calculate/estimate a target material's properties 
              15 x to test
                 o calls
                   o TRXTargInit
                   o TRXTargFree
                   o TRXTargShieldSlater
                 
   TRXTargComposition( t, projbase, flag)
                     o Decompose target material into constituents 
                  16 . to complete/test
                     . mixtures 
                     o calls:
                       o TRXProj* 
                       o TRXTargInit
                       o TRXTargComposition
                       
   TRXTargDup    ( tto, tfr, flag)
                 o Duplicate a target material
               7 x to test
                 o calls
                   o TRXTargInit
                   o TRXTargFree
                   o TRXTargDup
                 
   TRXTargFree   ( t, flag)
                 o Release a target material's resources
               3 x to test
                 o calls
                   o TRXTargInit
                   o TRXTargFree
                 
   TRXTargImportComposition( t, comp, flag)
                 o Import a target material composition
               5 x to test
                 o calls
                 
   TRXTargImportMaterial( t, material, flag)
                 o Import a target material name
               4 x to test
                 o calls
                 
   TRXTargInit   ( t, flag)
                 o Initialize a target material
               2 x to test
                 o calls
                 
   TRXTargInspect( t, flag)
                 o Inspect a target material
               1 x to test
                 o calls
                   o TRXTargInspect
                 
   TRXTargIsValid( t, flag)
                 o Verify a target material
               6 x to complete/test
                 ! check valid shells too,
                   to be ready for xs calc.
                 o calls

   TRXTargList   ( t, flag)
                 o List a target material
               9 x to test 
                 o to complete
                   o list all info
                 o calls
                   o TRXTargSpace
                   o TRXTargIsValid
                 
   TRXTargMatch  ( t1, t2, flag)
                 o Check if target materials match
              10 x to test
                 o calls
                   o TRXTargIsValid
                 
   TRXTargMerge  ( tto, tfr, flag)
                 o merge target materials
              7c ? to do
                 o calls
                 
   TRXTargMolWeight( t, flag)
                 o Target material's molweight et al
              11 x to test
                 o calls
                   o TRXTargMolWeight
                 
   TRXTargRead   ( t, filename, flag)
                 o Read a target material
              12 x to complete/test
                 o mixtures as unshelled materials
                 o dedicated converter cfg => n,l
                 o calls
                   o TRXTargInit
                   o TRXTargFree
                   o TRXTargImportMaterial
                   o TRXTargImportComposition
                 
   TRXTargShieldSlater( t, flag)
                 o Estimate target material's shielding constants by Slater's rule
              14 x to test
                 o calls
                   o 
                 
   TRXTargSpace  ( t, flag)
                 o Space occupied by a target material
               8 x to test
                 o calls
                   o TRXTargSpace
                 
   TRXTargWrite  ( t, path, flag)
                 o Write a target material
              13 x to test
                 o dedicated converter n,l => cfg
                 o better (tabulated) layout for cfg
                 o calls
                   o TRXTargIsValid


*/
                                      /* function prototypes */
struct SCMDERR TRXTargCalc        ( struct STRXTARG *, struct STRXPROJBASE *,  const char *, unsigned int );
struct SCMDERR TRXTargComposition ( struct STRXTARG *, struct STRXPROJBASE *, unsigned int );
struct SCMDERR TRXTargDup         ( struct STRXTARG *, struct STRXTARG *, unsigned int );
struct SCMDERR TRXTargFree        ( struct STRXTARG *, unsigned int );
struct SCMDERR TRXTargImportComposition( struct STRXTARG *, const char *, unsigned int );
struct SCMDERR TRXTargImportMaterial   ( struct STRXTARG *, const char *, unsigned int );
struct SCMDERR TRXTargInit        ( struct STRXTARG *, unsigned int );
struct SCMDERR TRXTargInspect     ( struct STRXTARG *, unsigned int );
struct SCMDERR TRXTargIsValid     ( struct STRXTARG *, unsigned int );
struct SCMDERR TRXTargList        ( struct STRXTARG *, unsigned int );
int            TRXTargMatch       ( struct STRXTARG *, struct STRXTARG *, unsigned int );
struct SCMDERR TRXTargMerge       ( struct STRXTARG *, struct STRXTARG *, unsigned int );
struct SCMDERR TRXTargMolWeight   ( struct STRXTARG *, unsigned int );
struct SCMDERR TRXTargRead        ( struct STRXTARG *, const char *, unsigned int );
struct SCMDERR TRXTargShieldSlater( struct STRXTARG *, int, unsigned int );
struct SCMDERR TRXTargSpace       ( struct STRXTARG *, unsigned int );
struct SCMDERR TRXTargWrite       ( struct STRXTARG *, const char *, unsigned int );


struct STRXTARG _TRXTargTest ( unsigned int, unsigned int );

                                      /** \struct STRXTARGBASE 
                                          \brief all defined materials 
                                      */
#if defined(OLDTARG)
struct STRXTARGBASE {
   struct List sLH;
   struct SCMDERR sCE;
};
#endif
struct STRXTARGBASE {
   unsigned int uiFlag;               /**< flag bits */
   size_t usztSpace;                  /**< space occupied (bytes) */
   int nT, mT;                        /**< current/max # of targets in base */
#define TRXTARGBASE_TARGINC 8         /* mT increment */   
   struct STRXTARG *psT;              /**< [nT/mT] targets */
};

                                      /* target slot increment macro */
#define TRXTARGBASEINCREMENT(psTB,sCE,gError) \
         if ( (psTB)->nT >= (psTB)->mT ) {\
            void *pv = NULL;\
            pv = realloc( (psTB)->psT, (ssize_t) ((psTB)->mT+TRXTARGBASE_TARGINC) * sizeof(*((psTB)->psT)) );\
            if ( pv == NULL ) {\
               (sCE).iRC = ERRTRXPROJ_SYS; goto gError;\
            }\
            (psTB)->psT = pv;\
            memset( (psTB)->psT + (psTB)->mT, 0, TRXTARGBASE_TARGINC * sizeof(*((psTB)->psT)));\
            (psTB)->mT += TRXTARGBASE_TARGINC;\
         }                                

/*
   Functions to handle target base
   -------------------------------

   TRXTargBaseCalc   ( tb, projbase, list, flag)
                     o Try calculate target properties 
                  11 x to test
                     o calls:
                       o TRXTargBaseLocate
                       o TRXTargCalc
                       o TRXTargFree
                       o TRXTargInit
                       
   TRXTargBaseDelete ( tb, list, flag)
                     o Delete listed targets from base 
                  12 . to test
                     o calls:
                       o TRXTargFree
                       
   TRXTargBaseFree   ( tb, flag)
                     o Free target base 
                   3 . to test
                     o calls:
                       o TRXTargFree
                       o TRXTargBaseInit
                       
   TRXTargBaseInit   ( tb, flag)
                     o Initialize target base
                   2 . to test
                     o calls:
                     
   TRXTargBaseInspect( tb, list, flag)
                     o Inspect a target base
                   1 x to test
                     o calls:
                       o TRXTargInspect
                       
   TRXTargBaseIsValid( tb, flag)
                     o Verifies validity of a target base
                   4 x to test
                     o calls:
                       o TRXTargIsValid
                       
   TRXTargBaseList   ( tb, list, flag) 
                     o List targets in base
                   6 x to test
                     o to complete
                     o calls:
                       o TRXTargList
                       
   TRXTargBaseLocate ( tb, name, flag)
                     o Locate a target material in base
                   7 . to test
                     o calls:
                     
   TRXTargBaseRead   ( tb, projbase, list, flag)
                     o Read target material files into target material base 
                   9 x to test
                     o calls:
                       o TRXTargBaseLocate
                       o TRXTargShellFromBase
                       o TRXTargRead
                       o TRXTargFree
                       o TRXTargInit
                       o TRXTargComposition
                       o TRXTargMolWeight
                       
   TRXTargBaseSpace  ( tb, flag) 
                     o Space occupied by a target material
                   5 x to test
                     o calls:
                       o TRXTargSpace
                       
   TRXTargBaseWrite  ( tb, path, list, flag) 
                     o Write target materials from target material base 
                  10 x to test
                     o calls:
                       o TRXTargWrite  

   TRXTargCompleteFromBase( t, tb, projbase, flag)
                     o Completes entries within a target material 
                     o obsolete
                     ? should refer to tb rather than projbase
                     o calls:
                       o TRXTargComposition
                       o TRXTargMolWeight
                       o TRXTargShellFromBase
                       
   TRXTargShellFromBase( t, tb, flag)
                     o Completes target material's shell information from base 
                   8 . to test
                       o parentage
                     o calls:
                       o TRXTargBaseLocate
                       o TRXTargShieldSlater
                       
   

*/
struct SCMDERR TRXTargBaseCalc   ( struct STRXTARGBASE *, struct STRXPROJBASE *, const char *, unsigned int );
struct SCMDERR TRXTargBaseDelete ( struct STRXTARGBASE *, const char *, unsigned int );
struct SCMDERR TRXTargBaseFree   ( struct STRXTARGBASE *, unsigned int );
struct SCMDERR TRXTargBaseInit   ( struct STRXTARGBASE *, unsigned int );
struct SCMDERR TRXTargBaseInspect( struct STRXTARGBASE *, const char *, unsigned int );
struct SCMDERR TRXTargBaseIsValid( struct STRXTARGBASE *, unsigned int );
struct SCMDERR TRXTargBaseList   ( struct STRXTARGBASE *, const char *, unsigned int );
struct STRXTARG *TRXTargBaseLocate ( struct STRXTARGBASE *, const char *, unsigned int );
struct SCMDERR TRXTargBaseRead   ( struct STRXTARGBASE *, struct STRXPROJBASE *, const char *, unsigned int );
struct SCMDERR TRXTargBaseSpace  ( struct STRXTARGBASE *, unsigned int );
struct SCMDERR TRXTargBaseWrite  ( struct STRXTARGBASE *, const char *, const char *, unsigned int );
/*
struct SCMDERR TRXTargCompleteFromBase   ( struct STRXTARG *, struct STRXTARGBASE *, struct STRXPROJBASE *, unsigned int );
*/
struct SCMDERR TRXTargShellFromBase      ( struct STRXTARG *, struct STRXTARGBASE *, unsigned int );

struct STRXTARGBASE _TRXTargBaseTest ( unsigned int );
 
                                      /* command processors */
int TRXCmdTarg(
                const char *, const char *, const char *, const char *,
                int32_t *, int32_t *, int32_t *, int32_t *, int32_t *,
                int32_t *, int32_t *
              );

int TRXCmdTargTest(
                const char *, int32_t *
              );

int TRXTargError( struct SCMDERR *, unsigned int );
/*
   --- error codes
*/
enum { ERRTRXTARG_SYS=1, 
       ERRTRXTARG_BADADDR,
       ERRTRXTARG_MISMATCHNAME,
       ERRTRXTARG_BADNAME,
       ERRTRXTARG_BADDENSITY,
       ERRTRXTARG_BADMOLWEIGHT,
       ERRTRXTARG_NOSUCHTARGET,
       ERRTRXTARG_NOFILE,
       ERRTRXTARG_CMDPARNOTYET,
       ERRTRXTARG_BADMATERIAL,
       ERRTRXTARG_NOSUCHMATERIAL,
       ERRTRXTARG_NOMOLWEIGHT,
       ERRTRXTARG_BADFILETYPE,
       ERRTRXTARG_BADFILEVERS,
       ERRTRXTARG_ITEMNOTYET,
       ERRTRXTARG_UNKNOWNTAG,
       ERRTRXTARG_SYNTAX,
       ERRTRXTARG_BADSHELL,
       ERRTRXTARG_BADFILELIST,
       ERRTRXTARG_BASEEMPTY,
       ERRTRXTARG_NAMEMISMATCH,
       ERRTRXTARG_COMPMISMATCH,  
       ERRTRXTARG_DENSMISMATCH,  
       ERRTRXTARG_MOLWMISMATCH,  
       ERRTRXTARG_WEIGHTMISMATCH,  
       ERRTRXTARG_NMOLMISMATCH,  
       ERRTRXTARG_NNUCMISMATCH,  
       ERRTRXTARG_SHELLMISMATCH,  
       ERRTRXTARG_CONSTITUENTMISMATCH,  
       ERRTRXTARG_BADCOMPOSITION,
       ERRTRXTARG_BADELEMENT,
       ERRTRXTARG_NOSUCHATOM,
       ERRTRXTARG_NOSUCHSHELL,
       ERRTRXTARG_BADNUMBER,
       ERRTRXTARG_BADPARENT,
       ERRTRXTARG_BADATOM,
       ERRTRXTARG_BADCOMPCHAR,
       ERRTRXTARG_BADCOMPNUM,
       ERRTRXTARG_BADMIXTURE,
       ERRTRXTARG_BADMIXTURESUM,
       ERRTRXTARG_BADMIXTUREREL,
       ERRTRXTARG_BADMIXTURESYNTAX,
       ERRTRXTARG_NOSHELLAVAIL,
       ERRTRXTARG_BADTARGLIST,
       ERRTRXTARG_NOPARENT,
       ERRTRXTARG_PARENTNOSHELL,
       ERRTRXTARG_READUNKNOWNTAG,
       ERRTRXTARG_READUNKNOWNTAGDATA,
       ERRTRXTARG_BADEKIN,
       ERRTRXTARG_NOTATOMIC,
};

#endif
