#ifndef TRXXS_H
#define TRXXS_H
/** \file
   Definitions for cross section tables.
*/
/*

   Copyright  1992-2011 M.Kraemer, GSI Darmstadt

   $Log: trxxs.h,v $
   Revision 1.3  2009/05/29 12:31:42  kraemer
   reworked target tables, projectiles

   Revision 1.2  2002/11/25 16:39:38  kraemer
   locate macros; outside and emfld pseudo-interactions

   Revision 1.1  2001/01/09 14:54:44  kraemer
   *** empty log message ***


*/

#include <limits.h>

#include "cmderr.h"
#include "trxconst.h"
#include "trxmmap.h"
#include "trxproj.h"
#include "trxtarg.h"

#define NEWXS
                                      /* Barkas formula */
#define BARKAS(Z0,beta2) ( (Z0) == 1 ) ? (Z0) : ( (Z0) * ( ( 1E0 - exp( -125E0 * sqrt(beta2) * pow( (Z0), -2E0/3E0 ) ) ) ) )
#define TRXXSHASWILD(str) \
   ( ( (str) != NULL ) && ( (strchr(str,'*') != NULL) || (strchr(str,'*') != NULL) ) )
                                      /* Common status and action flags */
                                      /* */
                                      /* masks */
#define TRXXS_MASKGLOBAL 0xFF000000   /* global, bits automatically passed through all function levels */
#define TRXXS_DEBUG      0x80000000   /* general check flag */
#define TRXXS_QUIET      0x40000000   /* suppress error messages */
#define TRXXS_NOMSG      TRXXS_QUIET
#define TRXXS_SHOWHEADER 0x01000000   /* for Inspect(), List(), ... : show at least structure headers, even if substructures are empty */
#define TRXXS_SHOWTABLES 0x02000000   /* for Inspect(), List(), ... : show tables */
#define TRXXS_DUPASREF   0x04000000   /* duplicate as a reference copy, i.e. no allocation */
#define TRXXS_NOERASE    0x08000000   /* do not erase input XS */

#define TRXXS_MASKFUNC   0x00FF0000   /* function specific flags */
#define TRXXS_MASKQUAL   0x0000FFFF   /* for additional qualifiers */



                                      /** \struct STRXXSTAG
                                          \brief Aux. structure to correlate tag string <=> ID.
                                      */ 
struct STRXXSTAGID { char *pcTag; int iTag; };

                                      /** \struct STRXXSCALC
                                          \brief Aux. structure to specify cross section calculation input.
                                      */ 
struct STRXXSCALC {
   double dRelChange;                 /**< minimum rel. change between xs table points */
   double dvELim[2];                  /**< range of primary's energies */
   double dvWLim[2];                  /**< range of secondary"s energies */
   double dvALim[2];                  /**< range of secondary's angles (deg) */
   double *pdE, *pdW, *pdA;           /**< list of table breakpoints (optional) */
   int nE, nW, nA;                    /**< # of table breakpoints */
   int iMethod;                       /**< cross section calculation method, ID */
   unsigned int uiFlag;               /**< flag bits */ 
#define TRXXSCALC_WABSOLUTE 0x00000001 /* for ions: W's are absolute, not relative to WNorm */ 
};    

                                      /* calc. methods */
enum { TRXXS_METHNONE=0, TRXXS_METHDEFAULT=TRXXS_METHNONE,
       TRXXS_METHBEA, TRXXS_METHRUDD, 
       TRXXS_METHHKS, TRXXS_METHCDWEIS, 
       TRXXS_METHREL,
       TRXXS_METHRBEB,
       TRXXS_METHBEB,
       TRXXS_METHEMP,
       TRXXS_METHRFORD,
       TRXXS_METHGSKI,
};
                                      /* function specific action flags */
#define TRXXSCALCLIMITSFROMSTR_E    0x00010000  
#define TRXXSCALCLIMITSFROMSTR_W    0x00020000  
#define TRXXSCALCLIMITSFROMSTR_A    0x00040000  
#define TRXXSCALCLIMITSFROMSTR_WABS 0x00080000  

/*
   TRXXSCalcFree         ( xscalc, flag )
                         o release calculation specs
                         x to test
   
   TRXXSCalcInspect      ( xscalc, flag )
                         o inspect calculation specs
                         x to test
   
   TRXXSCalcLimitsFromStr( xscalc, str, flag )
                         o determine calculation limits
                         x to test
   
   TRXXSCalcMethodFromStr( xscalc, str, flag )
                         o determine calculation method
                         x to test
   
*/
struct SCMDERR TRXXSCalcFree         ( struct STRXXSCALC *, unsigned int );
struct SCMDERR TRXXSCalcInspect      ( struct STRXXSCALC *, unsigned int uiFlag );
struct SCMDERR TRXXSCalcLimitsFromStr( struct STRXXSCALC *, const char *, unsigned int );
struct SCMDERR TRXXSCalcMethodFromStr( struct STRXXSCALC *, const char *, unsigned int );

/* 
   --- Cross sections for one primary energy 
*/
                                      /** \struct STRXXSE
                                          \brief Cross sections for a single primary energy.
                                      */ 
struct STRXXSE {
   double dE;                         /**< primary Energy,
                                           ions: MeV/u
                                           else: keV
                                      */
   double dEScaled;                   /**< scaled for interpolation */
   double dSigTot;                    /**< total cross section [cm**2] */
   double dSigTotScaled;              /**< scaled for interpolation */
   double dkm;                        /**< mean potential energy transfer */
   double dbeta2;                     /**< (v/c)**2 */
   double dgamma;                     /**< Lorentz factor */
   double dEkin;                      /**< mean kinetic energy of ejected secondaries [keV] */
   double dLET;                       /**< [] */
   double dWNorm;                     /* scale factor for secondary energies
                                         ions: binary peak 
                                      */
   double dvW[4];
   
   int  n1;                           /**< # of secondary energy/angle values */          
   double  *pd1;                      /**< [n1] secondary energy/angle values */
   double  *pdSig1;                   /**< [n1] SDCS */
   double  *pdCum1;                   /**< [n1] SDCS cumulated, normalized */
   int     *pn2;                      /**< [n1] # of secondary angle values */
   double **pd2;                      /**< [n1][n2[]] secondary angle values (cos, 1..-1)*/
   double **pdSig2;                   /**< [n1][n2[]] DDCS */
   double **pdCum2;                   /**< [n1][n2[]] DDCS cumulated, normalized */
#undef NEWIPOL
#if defined(NEWIPOL)
   int n1IPol;                        /**< # of secondary energy/angle values */          
   double  *pd1IPol;                  /**< [n1] secondary energy/angle values */
   double  *pvdCum1[2];               /**< [n1] SDCS cumulated, normalized */
#endif
   struct SCMDERR sCE;                /**< to keep error msg during read */ 
};
                                      /** \struct STRXXSSH
                                          \brief Cross sections for a single shell.
                                      */ 
struct STRXXSSH {
   unsigned int uiFlag;               /**< flag bits */   
#define TRXXS_EQUIVW 0x00000001       /* normalized W-tables are equivalent for all primary energies */
   int iSh;                           /**< actual shell number (0=all shells) */
   int nE;                            /**< # of primary energies */
   struct STRXXSE *psE;               /**< [nE] table of cross sections for primary energies */
   struct SCMDERR sCE;                /**< to keep error msg during read */ 
};


                                      /* common macro to find primary energy bin */
#if defined(NEWXS)

#define TRXXSLOCATEE(iE,psXSELo,psXSEHi,iLo,iHi,psSh,dEScaled)\
   iLo = 0; iHi = (psSh)->nE-1; iE = iLo;\
   for(;;) {\
      if ( ( iE = ( ((iLo)+(iHi)) >> 1 ) ) == iLo ) break;\
      if ( (dEScaled) == (psSh)->psE[iE].dEScaled ) break;\
      if ( (dEScaled) <  (psSh)->psE[iE].dEScaled ) { iHi = iE; }\
      else                                          { iLo = iE; }\
   }\
   psXSELo = (psSh)->psE + (iE);\
   (iE)++; iE = min( (iE), ((psSh)->nE-1) );\
   psXSEHi = (psSh)->psE + (iE);

#else
#endif                                  
                                      /* common macro to find SDCS quantity */
#if defined(NEWIPOL)

#define TRXXSLOCATE1(i1,d1,psELo,psEHi,iLo,iHi,ddLo,ddHi,psXS,ddEScaled,dq,dd)\
   iLo = 0; iHi = (psELo)->n1IPol-1; i1 = iLo;\
   for(;;) {\
      if ( ( i1 = ( ((iLo)+(iHi)) >> 1 ) ) == iLo ) break;\
      dd = ( (psEHi) -> pvdCum1[0][i1] - (psELo) -> pvdCum1[1][i1] ) * (ddEScaled) +\
                                         (psELo) -> pvdCum1[1][i1];\
      if ( (dq) == (dd) ) break;\
      if ( (dq)  < (dd) ) { iHi = i1; }\
      else                { iLo = i1; }\
   }\
   ddHi = ( (psEHi) -> pvdCum1[0][(i1)+1] - (psELo) -> pvdCum1[1][(i1)+1] ) * (ddEScaled) +\
                                            (psELo) -> pvdCum1[1][(i1)+1];\
   ddLo = ( (psEHi) -> pvdCum1[0][ i1   ] - (psELo) -> pvdCum1[1][ i1   ] ) * (ddEScaled) +\
                                            (psELo) -> pvdCum1[1][ i1   ];\
   d1   = ( (psELo) -> pd1IPol[i1+1] - (psELo) -> pd1IPol[i1  ] )\
        / ( (ddHi) - (ddLo) )\
        * ( (dq  ) - (ddLo) )\
        +   (psELo) -> pd1IPol[i1  ];                                  

#else

#define TRXXSLOCATE1(i1,d1,psELo,psEHi,iLo,iHi,ddLo,ddHi,psXS,ddEScaled,dq,dd)\
   iLo = 0; iHi = (psELo)->n1-1; i1 = iLo;\
   for(;;) {\
      if ( ( i1 = ( ((iLo)+(iHi)) >> 1 ) ) == iLo ) break;\
      dd = ( (psEHi) -> pdCum1[i1] - (psELo) -> pdCum1[i1] ) * (ddEScaled) +\
                                     (psELo) -> pdCum1[i1];\
      if ( (dq) == (dd) ) break;\
      if ( (dq)  < (dd) ) { iHi = i1; }\
      else                { iLo = i1; }\
   }\
   ddHi = ( (psEHi) -> pdCum1[(i1)+1] - (psELo) -> pdCum1[(i1)+1] ) * (ddEScaled) +\
                                        (psELo) -> pdCum1[(i1)+1];\
   ddLo = ( (psEHi) -> pdCum1[ i1   ] - (psELo) -> pdCum1[ i1   ] ) * (ddEScaled) +\
                                        (psELo) -> pdCum1[ i1   ];\
   d1   = (psELo) -> pd1[i1  ];\
   if ( ( (ddHi) - (ddLo) ) != 0.0 )\
      d1  += ( (psELo) -> pd1[i1+1] - (psELo) -> pd1[i1  ] )\
           / ( (ddHi) - (ddLo) )\
           * ( (dq  ) - (ddLo) )\
           ;                                  

#endif                                  

/*
   --- data base for one projectile/target/interaction combination
*/
                                      /* cross section dimension */ 
enum {
   TRXXS_TCS=0, TRXXS_SDCS=1, TRXXS_DDCS=2,
   TRXXS_MAXDIM
};

                                      /** \struct STRXXS
                                          \brief Cross sections for a specific p/t/ia.
                                      */ 
struct STRXXS {
   unsigned int uiFlag;               /**< flag bits */   
#define TRXXS_REFCOPY    0x00000004   /* reference copy, do not free() */
#define TRXXS_VALID      0x00000001   /* marks valid cross section */  
#define TRXXS_ENABLE     0x00000002   /* marks enabled cross section */  
            int iIA;                  /**< interaction type */
   size_t ulSpace;                    /**< space counter */
   int nDim;                          /**< actual cross section dimension (0/1/2 for total,single,double) */
   unsigned int uiDimBits;            /**< cross section dimension selected for output (0 == all) */
#define TRXXS_TCSBIT  (1L<<TRXXS_TCS) 
#define TRXXS_SDCSBIT (1L<<TRXXS_SDCS)
#define TRXXS_DDCSBIT (1L<<TRXXS_DDCS)
   char *pcInfile;                    /**< cross section input filename */
   char *pcIA;                        /**< interaction string (reference only, don't free() !) */
   char *pcDim;                       /**< dimension string (reference only, don't free() !) */
   char *pcColour;                    /**< colour string (reference only, don't free() !) */
   int  nSh;                          /**< # of shells to be considered,
                                           may be one more than natural shell numbers,
                                           to hold sum of all shells.
                                      */
   struct STRXXSSH *psSh;             /**< [nSh] shell specific cross sections
                                           [last=nSh-1] holds sum of all
                                      */
   struct STRXMMAP sM;                /**< for memory mapping */
   struct STRXTARG sT;                /**< specifies target properties */
   struct STRXPROJ sP;                /**< specifies projectile properties */
};

                                      /* enums */
                                      /* interaction types */     
#if defined(NEWXS)
enum {
                                      /* true interactions (table based) */ 
   TRXXS_NONE=0,
   TRXXS_ELASTIC,                     /*  */    
   TRXXS_IONIZATION, 
   TRXXS_EXCITATION,
   TRXXS_BREMS, 
   TRXXS_ANNIH,
   TRXXS_PHOTO,
   TRXXS_COMPTON,
   TRXXS_PAIR,
   TRXXS_MAXXS,                       /* max # of true interactions */  
                                      /* pseudo interactions */ 
                                      /* codes must be < 32 ! (bit positions in 32-bit word) */
   TRXXS_OUTSIDE=31,
   TRXXS_MAXPSEUDOXS,                 /* max # of true+pseudo interactions */  
   TRXXS_EMFLD=TRXXS_OUTSIDE-1,
   TRXXS_BOUNDARY=TRXXS_EMFLD-1,
   TRXXS_CUTOFF=TRXXS_BOUNDARY-1,
   
   TRXXS_BOR=INT_MAX-1, TRXXS_EOR    /* dummies for begin/end-of-run */
};
#else
#endif

#undef TRXXS_TOTALREACTION
#if defined(TRXXS_TOTALREACTION)
enum {
   TRXXS_NUCLREACTION=TRXXS_ANNIH,    /* 7 */ 
};
#endif

#define TRXXS_FILEVERSION 20080825

#define TRXXS_CHARS "???lixbafcp"

enum {
   TRXXS_BITOUTSIDE    = ( 1L << TRXXS_OUTSIDE ), 
   TRXXS_BITBOUNDARY   = ( 1L << TRXXS_BOUNDARY ), 
   TRXXS_BITCUTOFF     = ( 1L << TRXXS_CUTOFF ), 
   TRXXS_BITELASTIC    = ( 1L << TRXXS_ELASTIC ), 
   TRXXS_BITIONIZATION = ( 1L << TRXXS_IONIZATION ), 
   TRXXS_BITEXCITATION = ( 1L << TRXXS_EXCITATION ), 
   TRXXS_BITBREMS      = ( 1L << TRXXS_BREMS ), 
   TRXXS_BITANNIH      = ( 1L << TRXXS_ANNIH ), 
   TRXXS_BITEMFLD      = ( 1L << TRXXS_EMFLD ), 
};
enum {
   TRXXS_BITPHOTO      = ( 1L << TRXXS_PHOTO ), 
   TRXXS_BITCOMPTON    = ( 1L << TRXXS_COMPTON ), 
   TRXXS_BITPAIR       = ( 1L << TRXXS_PAIR ), 
};

                                      /* function action flags */ 
#define TRXXSCALCXX_ALLOWBADXS 0x00010000 /* do not report unsupported interaction as an error */
                                      
/*

   Functions to handle a single cross section
   ------------------------------------------

   TRXXSIATypeFromStr( xs, iastr, flag )
                     o interaction type from given string 
                  8a x to test
                     o calls:
                   
   TRXXSIADimFromStr ( xs, iastr, flag )
                     o interaction dimension from given string 
                  8b x to test
                     o calls:
                   
   TRXXSColFromIAType( xs, flag )
                     o colour string from stored interaction type 
                  8e o to test (formally)
                     o calls:
                   
   TRXXSStrFromIAType( xs, flag )
                     o interaction string from stored type 
                  8c x to test
                     ? include dimension info
                     o calls:
                   
   TRXXSStrFromIADim ( xs, flag )
                     o interaction string from stored dimension  
                  8d x to test
                     o calls:
                   

   TRXXSCalc       ( xs, xscalc, flag )
                     Cross section calculation steering function
                14 o to complete
                   o calls:
                     o TRXXSCalcEL
                     o TRXXSCalcEX
                     o TRXXSCalcEI
                     o TRXXSCalcEB
                     o TRXXSCalcII   
                     o TRXXSCalcIX
                     o TRXXSCalcXP
                     o TRXXSCalcXC
                       ! CumA < 0, >1,
                     o TRXXSCalcXF
                     o TRXXSCalcXL
                     o TRXXSCalcPL
                     o TRXXSCalcPX
                     o TRXXSCalcPI
                     o TRXXSCalcPB
                     o TRXXSCalcPA
                                            

   TRXXSCalcEL     ( xs, xsc, flag )
               14a . to complete/test
                   o respect pre-allocated tables
                   o E > limit: screened Rutherford/Mott
                   o E < limit: material specific fits
                   o calls:
                      
   TRXXSCalcEX     ( xs, xsc, flag ) 
               14b . to complete/test
                   . respect pre-allocated tables
                   o from old cross sections
                     o H2O
		                     o rise < 10eV realistic ?
                     o N2
                       o from old formula
                       o store scaled like fit above
                     o C2H4
                       o from H2O
                       o check w/ W-value and stopping power
                     o C
                       o pickup experimental (?) values
                       o fit 
                     ? Al
                   o relativistic 
                   o consult IAEA
                   o calls:
                      
   TRXXSCalcEI     ( xs, xsc, flag )
               14c . to complete/test
                   o respect pre-allocated tables
                   o
                     o H2O
		                     o total xs from subshells
		                     o km from subshells
                     o N2
                     o C2H4
                     o C
                     ? Al
                   o calls:



   TRXXSCalcII     ( xs, xsc, flag )
               14d . to complete / to test
                   o respect pre-allocated tables
                   o possibility to select E, W, A, nE, nW, nA
                   o theories/methods
                     o BEA
                     o Hansen/Kocbach
                       o empirical 
                       o possibly as user plugin
                     o CDW/EIS  
                     o relativistic
                       o from Moeller
                       o from BEA
                       o from Hansen
                       o Jakubassa
		                   o Wmin = 0
                   o calls:
                      
   TRXXSCalcIX     ( xs, xsc, flag )
               14e . to complete/test
                   o respect pre-allocated tables
                   o calls:
                      
                       
   TRXXSCalcXF     ( psXS, psP, psT, psXSC, uiFlag )
               14f o to do
                   o dSigma/dOmega of photoelectrons 
		                 o Davisson eq 43 / GW eq 10
		                   (cumulated distrib. valid only for v/c = 1, Z/137 << 1)
                                 ! better angular grid: more dense for angles => 0  
		                 ? < 8 keV: LCAO, Yeh/Lindau At Data NDT 1985 32:1-55
		                 ? own calculation, e.g.: Hulme Proc Roy Soc 133A 381 (1931)
		                 o total cross section
		                   o Davisson Eqs 37 (>2 MeV), 38 ( <2 MeV)
		                     problem for compounds: which Z ???
		                     high energy: only K-Shell => highest Z
                                   ! some E**x scaling  
		                   o Storm/Israel NDT 7 (1970) 565
		                     o 1..200 keV: Brysk/Zerby PR 171 (1968) 292
		                     o > 200 keV : Schmickley/Pratt PR 164 (1967) 104
		                   ? own calculation (Veigele, low energy cross sections)
		                 o Auger electrons:
		                   o H2O: 1 per PE, E=Ek-Elast
		                   o proper fluorescent/Auger yields    
                   o calls:
                      
   TRXXSCalcXC     ( psXS, psP, psT, psXSC, uiFlag ) 
               14g o to complete / test
                   o respect pre-allocated tables
                   ? binding energy
                   o evtl more energy steps for high energies
		                 o strange steps in cumulated arrays for very low energies
		                 o E <= 0.01 keV: negative CumA
		                 o form factors for low energies / shell dependencies
                   o scaling to reduce # of table points
                   o calls:
                      
                      
   TRXXSComplete   ( xs, flag )
                   o complete cross section tables 
                11 . to complete / test (real example)
                   . scaling/norm 
                   ! consistency check (W,A)
                   . sum of shells, if necessary (to test)
                   . complete km (to test)
                   o calls:
                   
   TRXXSDisable    ( xs, flag )
                   o Disable cross section
                7a x to test
                   o calls:

   TRXXSDup        ( xsto, xsfr, flag )
                   o Duplicate cross sections (real copy w/ allocation)
                 6 x to test
                   o how to support memory mapping ?
                   ? copy infile too
                   o calls:
                     o TRXXSFree()
                     o TRXXSInit()
                     o TRXXSSpace()
                     o TRXProj*()
                     o TRXTarg*()
                     
   TRXXSEnable     ( xs, flag )
                   o Enable cross sections
                7b x to test
                   o calls:

   TRXXSExportgd   ( xs, prefix, flag )
                   o Export cross sections, gd format
               10a . to complete / to test
                   . TCS,SDCS,DCCS
		                 . add some xsect specific stuff 
                     o scaling, acos()
                     o xlo, ylo
                   . latest specs
                   . additional/derived quantities
		                 . colours / projectile types
		                   o i: red
		                   o x: green
		                   o l: blue
		                   o b:
		                   o a:                        
                   o calls:
		                   o TRXXSIsValid()
		                   o TRXXSStr*()

   TRXXSFree       ( xs, flag )
                   o Release cross sections resources
                 3 x to test
		                 o account for mmap()ed files
                   o calls:
		                   o TRXXSInit() 
                     o TRXProj*()
                     o TRXTarg*()
                   
   TRXXSInit       ( xs, flag )
                   o Initialize cross section   
                 2 x to test
		                 o account for mmap()ed files
                   o calls:
                     o TRXProjInit
                     o TRXTargInit
                   
   TRXXSInspect    ( xs, flag )
                   o Inspect cross sections 
                 1 x to test
                   o calls:
                     o TRXProjInspect
                     o TRXTargInspect
                   
   TRXXSIsEquivW   ( xs, eps, flag ) 
                   o tests (and flags) if cross sections have equivalent W-tables
                12 x to test
                   o calls:

   TRXXSIsValid    ( xs, flag ) 
                   o Validate cross sections 
                 4 x to test
                   ? set validate flag ?
                   . more diagnostic output
                     . tests:
                       x nE > 0; 
                       x sigma >= 0  
                       x 0 < cum < 1;
                       x monotonic 
                   o calls:
                     o TRXProjIsValid
                     o TRXTargIsValid

   TRXXSList       ( xs, flag ) 
                   o List some cross section properties (only non-array stuff)                 
                 9 x to complete / to test
                   o improve layout
                   o calls:
                     o TRXXSStrFromIAType()
                     o TRXXSIsValid()
                      
   TRXXSRead       ( xs, filename, flag )
                     read cross section file
                11 . to test 
                     o more extensive tests todo
                   ? memory mapping for binaries
                   x Complete() to be done by caller 
                   ! uFileMemLines() 
                     o empty line problem to be fixed
                     o too slow
                   . even w/ utxt2mem() slow
                   . check duplicate values: Sh, E, W
                   o calls: 
                     o TRXXSFree()
                     o TRXXSInit()
                     o TRXXSError()
                     o TRXTarg*()
                     o TRXProj*()
                                        
   TRXXSSpace      ( xs, flag ) 
                     Estimate space for cross sections
                 5 x to test
                   o exact numbers still to check
                   ! account for refcopy
                   o calls
                     o TRXTarg*()
                     o TRXProj*()
                    
   TRXXSWrite      ( xs, prefix, flag )
                   o Write cross sections, native format
               10b . to complete / to test   
                   ? memory mapping
                   ? binary
                   . latest specs
                   . additional/derived quantities
                   o calls:
                     o TRXXSValid()    
		                   o TRXXSStr*()


*/

                                      /* function prototypes */

struct SCMDERR TRXXSIATypeFromStr( struct STRXXS *, const char *, unsigned int );
struct SCMDERR TRXXSIADimFromStr ( struct STRXXS *, const char *, unsigned int );
struct SCMDERR TRXXSColFromIAType( struct STRXXS *, unsigned int );
struct SCMDERR TRXXSStrFromIAType( struct STRXXS *, unsigned int );
struct SCMDERR TRXXSStrFromIADim ( struct STRXXS *, unsigned int );

struct SCMDERR TRXXSCalc  ( struct STRXXS *, struct STRXXSCALC *, unsigned int );
struct SCMDERR TRXXSCalcXC( struct STRXXS *, struct STRXPROJ *, struct STRXTARG *, struct STRXXSCALC *, unsigned int );
struct SCMDERR TRXXSCalcXF( struct STRXXS *, struct STRXPROJ *, struct STRXTARG *, struct STRXXSCALC *, unsigned int );
struct SCMDERR TRXXSCalcII( struct STRXXS *, struct STRXXSCALC *, unsigned int );
struct SCMDERR TRXXSCalcIX( struct STRXXS *, struct STRXXSCALC *, unsigned int );
struct SCMDERR TRXXSCalcEI( struct STRXXS *, struct STRXXSCALC *, unsigned int );
struct SCMDERR TRXXSCalcEL( struct STRXXS *, struct STRXXSCALC *, unsigned int );
struct SCMDERR TRXXSCalcEX( struct STRXXS *, struct STRXXSCALC *, unsigned int );

struct SCMDERR TRXXSComplete( struct STRXXS *, unsigned int );
struct SCMDERR TRXXSDisable ( struct STRXXS *, unsigned int );
struct SCMDERR TRXXSDup     ( struct STRXXS *, struct STRXXS *, unsigned int );
struct SCMDERR TRXXSEnable  ( struct STRXXS *, unsigned int );
struct SCMDERR TRXXSExportgd( struct STRXXS *, const char *, unsigned int );
struct SCMDERR TRXXSFree    ( struct STRXXS *, unsigned int );
struct SCMDERR TRXXSInit    ( struct STRXXS *, unsigned int );
struct SCMDERR TRXXSInspect ( struct STRXXS *, unsigned int );
#if defined(NEWIPOL)
struct SCMDERR TRXXSIntPolSetup ( struct STRXXS *, unsigned int );
#endif
struct SCMDERR TRXXSIsEquivW( struct STRXXS *, double, unsigned int );
struct SCMDERR TRXXSIsValid ( struct STRXXS *, unsigned int );
struct SCMDERR TRXXSList    ( struct STRXXS *, unsigned int );
struct SCMDERR TRXXSRead    ( struct STRXXS *, const char *, unsigned int );
struct SCMDERR TRXXSSpace   ( struct STRXXS *, unsigned int );
struct SCMDERR TRXXSWrite   ( struct STRXXS *, const char *, unsigned int );

struct STRXXS _TRXXSTestTCS ( int, unsigned int );             
struct STRXXS _TRXXSTestSDCS( unsigned int );             
struct STRXXS _TRXXSTestDDCS( unsigned int );             

struct STRXXS _TRXXSTestL( unsigned int );             
/*
   --- for one projectile
*/

                                      /** \struct STRXXSPROJ
                                          \brief Cross sections for a specific projectile.
                                      */ 
struct STRXXSPROJ {
   size_t ulSpace;                    /**< space counter */
   double dECutoff;                   /**< cutoff energy, from dissipative xs */
   struct STRXPROJ sP;                /**< specifies projectile properties */
#if defined(NEWXS)
   struct STRXXS svXS[TRXXS_MAXXS][TRXXS_MAXDIM]; /**< [][0/1/2] cross sections */
#else
#endif
};

/*  
   --- function action flags
*/


/*

   Functions to handle cross sections for one projectile
   -------------------------------------------------

   TRXXSProjCalc       ( xsp, targ, ia, calc, flag )
                       o Calculate projectile cross sections for selected T/IA combination.
                    14 . to complete
                       x dissipative cutoff
                       o calls:
                         o TRXXSCalc
                         o TRXXSIActFromStr
                         o TRXXSProjCutoff
                         
   TRXXSProjCutoff     ( xsp, flag )
                       o Determine cutoff energy from all dissipative cross sections.
                     9 x to test
                       o calls:
			   
   TRXXSProjDelete     ( xsp, ia, flag )
                       o Delete projectile cross section for specified interactions.
                    10 x to test
                       o calls:
                         o TRXXSProjCutoff
                         o TRXXSFree
                         o TRXXSIActFromStr
                         
   TRXXSProjDisable    ( xsproj, ia, flag )
                       o Disable specified projectile cross section.
                   11a x to test
                       o calls:
                         o TRXXSDisable
                         o TRXXSIActFromStr
                         
   TRXXSProjDup        ( xsprojto, xsprojfr, flag )
                       o Duplicate projectile cross section.
                     8 x to test
                       o calls:
                         o TRXProjDup
                         o TRXXSDup()
                         o TRXXSIsValid()
                         
   TRXXSProjEnable     ( xsp, ia, flag )
                       o Enable projectile cross section for specified interactions. 
                   11b x to test
                       o calls:
                         o TRXXSEnable
                         o TRXXSIActFromStr
                            
   TRXXSProjExportgd   ( xsproj, prefix, ia, flag )
                       o Export projectile cross section for specified interactions, gd format.
                    7a x to test
                       o calls:
		                       o TRXXSIsValid()
                         o TRXXSExportgd
                         o TRXXSIActFromStr
                         
   TRXXSProjFree       ( xsproj, flag )
                       o Release projectile cross section resources.
                     3 x to test
		                     o consider ref copy ?
                       o calls:
                         o TRXXSFree
                         o TRXProjFree
                         o TRXXSProjInit();
                         
   TRXXSProjInit       ( xsproj )
                       o Initialize projectile cross section.
                     2 x to test
                       o calls:
                         o TRXProjInit
                         o TRXXSInit
                       
   TRXXSProjInspect    ( xsproj, ia, flag )
                       o Inspect projectile cross section for specified interactions.
                     1 x to test
                       o calls:
                         o TRXProjInspect
                         o TRXXSIActFromStr
                         o TRXXSInspect
                       
   TRXXSProjIsValid    ( xsproj, flag )
                     4 x to test
                       o calls:
                         o TRXXSIsValid
                         o TRXProjIsValid()
                       
   TRXXSProjList       ( xsproj, ia, flag )
                       o List projectile cross section for specified interactions.
                     6 x to test
                       o calls:
                         o TRXXSProjSpace
                         o TRXXS*()
			                      o TRXProjSpace()
                       
   TRXXSProjMerge      ( xsprojto, ia, xsprojfr, flag )
                       o Merge projectile cross sections.
                    13 x to complete
                       ? a "noreplace"-flag 
                       x dissipative cutoff
                       o calls:
                         o TRXProj*()
                         o TRXXSProjXSAdd
                         

   TRXXSProjSpace      ( xsp, flag ) 
                       o count space
                     5 x to test
		                     o exact numbers still to check
                       o calls
                         o TRXXSSpace
                    
   TRXXSProjWrite      ( xsproj, prefix, ia, flag )
                       o Write projectile cross section for specified interactions.
                    7b o to complete/test
                       x calls:
                         o TRXXSIActFromStr
                         o TRXXSWrite
                         o TRXXSIsValid
                       
   TRXXSProjXSAdd      ( xsp, xs, flag ) 
                       o Add a cross section to projectile cross sections,
                         xs is erased.
                    12 x to test
                       ? a "no-erase" flag
                       o calls
                         o TRXXS*() 
                         o TRXProjMatch() 
                         o TRXProjCutoff()
*/
                                      /* function prototypes */
struct SCMDERR TRXXSProjCalc    ( struct STRXXSPROJ *, struct STRXTARG *, const char *, struct STRXXSCALC *, unsigned int );
struct SCMDERR TRXXSProjCutoff  ( struct STRXXSPROJ *, unsigned int );
struct SCMDERR TRXXSProjDelete  ( struct STRXXSPROJ *, const char *, unsigned int );
struct SCMDERR TRXXSProjDisable ( struct STRXXSPROJ *, const char *, unsigned int );
struct SCMDERR TRXXSProjDup     ( struct STRXXSPROJ *, struct STRXXSPROJ *, unsigned int );
struct SCMDERR TRXXSProjEnable  ( struct STRXXSPROJ *, const char *, unsigned int );
struct SCMDERR TRXXSProjExportgd( struct STRXXSPROJ *, const char *, const char *, unsigned int );
struct SCMDERR TRXXSProjFree    ( struct STRXXSPROJ *, unsigned int );
struct SCMDERR TRXXSProjInit    ( struct STRXXSPROJ *, unsigned int );
struct SCMDERR TRXXSProjInspect ( struct STRXXSPROJ *, const char *, unsigned int );
struct SCMDERR TRXXSProjIsValid ( struct STRXXSPROJ *, unsigned int );
struct SCMDERR TRXXSProjList    ( struct STRXXSPROJ *, const char *, unsigned int );
struct SCMDERR TRXXSProjMerge   ( struct STRXXSPROJ *, const char *, struct STRXXSPROJ *, unsigned int );
struct SCMDERR TRXXSProjSpace   ( struct STRXXSPROJ *, unsigned int );
struct SCMDERR TRXXSProjWrite   ( struct STRXXSPROJ *, const char *, const char *, unsigned int );
struct SCMDERR TRXXSProjXSAdd   ( struct STRXXSPROJ *, struct STRXXS *, unsigned int );

struct STRXXSPROJ _TRXXSProjTest( unsigned int );             


/*
   --- Cross sections for one material/target
*/
                                      /** \struct STRXXSTARG
                                          \brief Cross sections for a specific target.
                                      */ 
struct STRXXSTARG {
   struct STRXTARG sT;                /**< specifies target properties */
   unsigned int uiFlag;               /**< flag bits */ 
   size_t ulSpace;                    /**< space counter */

#define NEWXST
#if defined(NEWXST)

   int nXSP, mXSP;                    /**< current, max # of projectiles */
#define TRXXSTARG_INCPROJ 4           /* mXSP increment */   
   struct STRXXSPROJ *psXSP;          /**< [mXSP] */
   int iT, nT;                        /**< index/max # of target constituents (for mixtures) */
   struct STRXXSTARG *psXST;          /**< [nT] (for mixtures, else NULL) */

#else
#endif
};

/*
   Functions to handle cross section base
   --------------------------------------

   TRXXSTargCalc     ( xst, proj, iastr, xsc, flag )
                     o Calculate target cross sections for selected P/T/IA combination
                  14 o to test
                     o to complete: 
                       . mixtures
                       . wildcard/multiple ions
                       ? free old allocs ?
                       ? wildcards for projectile
                         (does it make sense for multiple ionic projectiles ?) 
                     o calls:
                       o TRXXSProjInit
                       o TRXXSProjDelete
                       o TRXXSProjCalc
		      
   TRXXSTargCollect  ( xst, flag)
                     o recursively setup target cross sections for mixtures
                   5 x to test
                     o calls
                       o TRXXSTargInit
                     
   TRXXSTargDelete   ( xst, proj, iastr, flag )
                     o Delete target cross sections 
                  10 x to test
		                   o calls:
		                     o TRXProj*()
		                     o TRXXSProjDelete
                       o TRXXSTargDelete
		              
   TRXXSTargDisable  ( xst, proj, iastr, flag )
                     o Disable target cross sections (wildcards).
                 11a x to test
		                   o calls
		                     o TRXProj*()
		                     o TRXXSProjDisable()
                       o TRXXSTargDisable
		      
   TRXXSTargDup      ( xstto, xsfr, flag )
                     o Duplicate target cross sections
                   9 x to test
		                   o calls:
		                     o TRXTarg*()
                       o TRXXSTargInit();
                       o TRXXSTargFree();
                       o TRXXSTargDup
                       o TRXXSTargSpace
                       o TRXXSProj*();
		      
   TRXXSTargEnable   ( xst, proj, iastr, flag )
                     o Enable target cross sections
                 11b x to test
		                   o calls 
                       o TRXProj*()
		                     o TRXXSProjEnable()
                       o TRXXSTargEnable
		     
   TRXXSTargExportgd ( xst, prefix, proj, iastr, flag )
                     o Export target cross sections, gd format
                  8a x to test
		                   o calls:
                       o TRXProj*()
                       o TRXXSProj*()
                       o TRXXSTargExportgd
		       
   TRXXSTargFree     ( xst, flag )
                     o release all resources for target cross sections 
                   3 x to test
		                   o calls:
                       o TRXTarg*()                                   
                       o TRXXSProj*();
                       o TRXXSTargInit();
                       o TRXXSTargFree                                   
		     
   TRXXSTargInit     ( xst, flag )
                     o Initialize target cross sections
                   2 x to test
		                   o calls:
                       o TRXTarg*()                                   
		     
   TRXXSTargInspect  ( xst, proj, iastr, flag )
                     o Inspect target cross sections
                   1 x to test
		                   o calls:
		                     o TRXTarg*()
                       o TRXProj*()
                       o TRXXSProjInspect();
                       o TRXXSTargInspect

   TRXXSTargIsValid  ( xst, flag)
                     o Validate target cross sections
                   4 x to test
                     . test target too   
		                   o calls:
                       o TRXTargIsValid
		                     o TRXXSProjIsValid
                       o TRXXSTargIsValid
		       
   TRXXSTargList     ( xst, proj, iastr, flag )	
                     o List target cross sections
                   7 x to test
                     o to complete: 
                       o improve layout
                     o calls:
                       o TRXProj*()
		                     o TRXXSProjList()
		                     o TRXTarg*()
                       o TRXXSTargList
                       
   TRXXSTargMerge    ( xstto, proj, iastr, xstfr, flag )
                     o Merge (replace) target cross sections 
                  13 x to test
                     ? multiple ions: what if projectile slots mismatch?
                     ? duplicate XST entries too?
                     ? allow adding constituent cross sections to mixtures 
		                   o calls:
		                     o TRXTarg*()
		                     o TRXXSProj*()
                       o TRXXSTargSpace
		       
   TRXXSTargSpace    ( xst, flag ) 
                     o count space for target cross sections
                   6 x to test
                     o to complete:
		                     o exact numbers still to verify
                     o calls
                       o TRXTargSpace()
		                     o TRXXSProjSpace()
                       o TRXXSTargSpace
                    
   TRXXSTargWrite    ( xst, prefix, proj, iastr, flag )
                     o Write target cross sections
                  8b x to test
		                   o consider swap for binary data
		                   o calls:
                       o TRXProj*()
                       o TRXXSProjWrite();
                       o TRXXSProjIsValid
                       o TRXXSTargWrite
		     
   TRXXSTargXSAdd    ( xst, xs, flag )
                     o Add cross sections (erases input xs, if not protected)
                  12 x to test
                     ? allow adding constituent cross sections to mixtures 
                     o calls
                       o TRXXSIsvalid
                       o TRXXSInit
                       o TRXXSProjInit
                       o TRXXSProjXSAdd
                       o TRXXSTargSpace
*/
                                      /* function prototypes */
struct SCMDERR TRXXSTargCalc    ( struct STRXXSTARG *, struct STRXPROJ *, const char *, struct STRXXSCALC *, unsigned int );
struct SCMDERR TRXXSTargCollect ( struct STRXXSTARG *, unsigned int );
struct SCMDERR TRXXSTargDelete  ( struct STRXXSTARG *, struct STRXPROJ *, const char *, unsigned int );
struct SCMDERR TRXXSTargDisable ( struct STRXXSTARG *, struct STRXPROJ *, const char *, unsigned int );
struct SCMDERR TRXXSTargDup     ( struct STRXXSTARG *, struct STRXXSTARG *, unsigned int );
struct SCMDERR TRXXSTargEnable  ( struct STRXXSTARG *, struct STRXPROJ *, const char *, unsigned int );
struct SCMDERR TRXXSTargExportgd( struct STRXXSTARG *,  const char *, struct STRXPROJ *, const char *, unsigned int );
struct SCMDERR TRXXSTargFree    ( struct STRXXSTARG *, unsigned int );
struct SCMDERR TRXXSTargInit    ( struct STRXXSTARG *, unsigned int );
struct SCMDERR TRXXSTargInspect ( struct STRXXSTARG *, struct STRXPROJ *, const char *, unsigned int );
struct SCMDERR TRXXSTargIsValid ( struct STRXXSTARG *, unsigned int );
struct SCMDERR TRXXSTargList    ( struct STRXXSTARG *, struct STRXPROJ *, const char *, unsigned int );
struct SCMDERR TRXXSTargMerge   ( struct STRXXSTARG *, struct STRXPROJ *, const char *, struct STRXXSTARG *, unsigned int );
struct SCMDERR TRXXSTargRead    ( struct STRXXSTARG *, const char *, unsigned int );
struct SCMDERR TRXXSTargSpace   ( struct STRXXSTARG *, unsigned int );
struct SCMDERR TRXXSTargWrite   ( struct STRXXSTARG *, const char *, struct STRXPROJ *, const char *, unsigned int );
struct SCMDERR TRXXSTargXSAdd   ( struct STRXXSTARG *, struct STRXXS *, unsigned int );

struct STRXXSTARG _TRXXSTargTest       ( unsigned int );             
struct STRXXSTARG _TRXXSTargTestMixture( unsigned int );             

/*
   --- complete cross section data base 
*/
                                      /* */
                                      /** \struct STRXXSBASE
                                          \brief Cross section data base.
                                      */ 
struct STRXXSBASE {
   size_t ulSpace;                    /**< space counter */
   int nT, mT;                        /**< current/max # of targets in base */
#define TRXXSTARGBASE_INC 8           /* mT increment */   
   struct STRXXSTARG *psXST;          /**< [nT/mT] targets */
};
                                      /* list empty cross section slots too */
#define TRXXSBASEINSPECT_EMPTY 0x00010000
/*
   Functions to handle cross section base
   --------------------------------------
   
   TRXXSBaseCalc     ( xsbase,  projbase, targbase, proj, targ, ia, xsc, flag )
                     o Calculate internal cross sections for selected P/T/IA combination.
                   8 x to test
		                   o calls
                       o TRXXSTarg*()
                       o TRXTarg*()
                       o TRXProj*()
                       o TRXXSBaseTargAdd()
                       o TRXXSBaseTargLocate                 

   TRXXSBaseDelete   ( xsbase, projbase, projstr, targstr, iastr, flag)
                     o Delete cross sections for selected P/T/IA combinations.
                  11 x to test
		                   o calls
                       o TRXXSTarg*()
		       
   TRXXSBaseDisable  ( xsbase, proj, targ, ia, flag )
                     o Disable cross sections for selected P/T/IA combinations (wildcards).
                 12a x to test
		                   o calls
                       o TRXXSTarg*( )
		        
   TRXXSBaseEnable   ( xsbase, proj, targ, ia, flag )
                     o Enable cross sections for selected P/T/IA combinations (wildcards).
                 12b x to test
		                   o calls
                       o TRXXSTarg*( )
		       
   TRXXSBaseExportgd ( xsbase, prefix, proj, targ, ia, flag )
                     o Export cross sections for selected P/T/IA combinations. gd format.
                   7 x to test
		                   o calls
                       o TRXXSTarg*()
		                     o TRXXSBaseIsValid()
		       
   TRXXSBaseFree     ( xsbase, flag )
                     o Release cross section data base (all resources).
                   3 x to test
		                   o calls
                       o TRXXSBaseInit()
		                     o TRXXSTargFree()
		       
   TRXXSBaseInit     ( xsbase, flag )
                     o Initialize a cross section data base.
                   2 x to test
		                   o calls

   TRXXSBaseInspect  ( xsbase, proj, targ, ia, flag )
                     o Inspect cross sections for selected P/T/IA combinations 
                   1 x to test
		                   o calls
		                     o TRXXSTargInspect()

   TRXXSBaseIsValid  ( xsbase, flag )
                     o Validate cross section data base.
                   4 x to test
                     ? set a flag bit
		                   o calls
                       o TRXXSTargIsValid()
		        
   TRXXSBaseList     ( xsbase, proj, targ, ia, flag)
                     o List some properties of cross sections for selected P/T/IA combinations.
                  10 x to test
		                   o better layout
		                   o calls
                       o TRXXSBaseSpace() (should not be called here, xsb read-only!)
                       o TRXXSTargList()
		       
   TRXXSBaseRead     ( xsbase, filelist, flag )
                     o Read cross sections form specified files.
                  15 x to test
                     ? filelist expansion by caller
                     . rework complete/target logic
		                   o calls
                       o TRXTarg*()
                       o TRXXS*()
                       o TRXXSBaseXSAdd
                       o TRXXSBaseSpace                

   TRXXSBaseSpace    ( xsbase, flag) 
                     o count space
                   9 x to test
                     o calls
                       o TRXXSTargSpace
                    
   TRXXSBaseTargLocate( xsbase, name, flag )
                     o Locate specific target in cross section data base.
                   5 x to test
                     o calls
			
   TRXXSBaseWrite    ( xsbase, prefix, proj, targ, ia, flag )
                     o Write cross sections for selected P/T/IA combinations, native format.
                  14 x to test
		                   o calls
                       o TRXXSTargWrite()
		                     o TRXXSBaseIsValid()
                    
   TRXXSBaseXSAdd    ( xsbase, xs, flag )
                     o add cross sections to cross section base (erase xs)
   		             13 x to test
                     ? option to keep xs 
		                   o calls
                       o TRXXS*()
                       o TRXXSBaseTargLocate
                       o TRXXSTarg*()
		       
   TRXXSBaseXSTargAdd( xsbase, xst, flag )
                     o Add target cross sections (erased) to cross section data base 
   		              6 x to test
                     ? optionally don't erase xst
		                   o calls
                       o TRXXSBaseTargLocate
                       o TRXXSTarg*()
		       
*/
                                      /* function prototypes */
struct SCMDERR TRXXSBaseCalc     ( struct STRXXSBASE *, struct STRXPROJBASE *, struct STRXTARGBASE *,
                                   const char *, const char *, const char *, 
                                   struct STRXXSCALC *, unsigned int );
struct SCMDERR TRXXSBaseDelete   ( struct STRXXSBASE *, struct STRXPROJBASE *, const char *, const char *, const char *, unsigned int );
struct SCMDERR TRXXSBaseDisable  ( struct STRXXSBASE *, struct STRXPROJBASE *, const char *, const char *, const char *, unsigned int );
struct SCMDERR TRXXSBaseEnable   ( struct STRXXSBASE *, struct STRXPROJBASE *, const char *, const char *, const char *, unsigned int );
struct SCMDERR TRXXSBaseExportgd ( struct STRXXSBASE *, struct STRXPROJBASE *, const char *, const char *, const char *, const char *, unsigned int );
struct SCMDERR TRXXSBaseFree     ( struct STRXXSBASE *, unsigned int );
struct SCMDERR TRXXSBaseInit     ( struct STRXXSBASE *, unsigned int );
struct SCMDERR TRXXSBaseInspect  ( struct STRXXSBASE *, struct STRXPROJBASE *, const char *, const char *, const char *, unsigned int );
struct SCMDERR TRXXSBaseIsValid  ( struct STRXXSBASE *, unsigned int );
struct SCMDERR TRXXSBaseList     ( struct STRXXSBASE *, struct STRXPROJBASE *, const char *, const char *, const char *, unsigned int );
struct SCMDERR TRXXSBaseRead     ( struct STRXXSBASE *, struct STRXTARGBASE *, struct STRXPROJBASE *, const char *, unsigned int );
struct SCMDERR TRXXSBaseSpace    ( struct STRXXSBASE *, unsigned int );
struct SCMDERR TRXXSBaseWrite    ( struct STRXXSBASE *, struct STRXPROJBASE *, const char *, const char *, const char *, const char *, unsigned int );
struct SCMDERR TRXXSBaseXSTargAdd( struct STRXXSBASE *, struct STRXXSTARG *, unsigned int );
struct SCMDERR TRXXSBaseXSAdd    ( struct STRXXSBASE *, struct STRXXS *, unsigned int );

struct STRXXSTARG *TRXXSBaseTargLocate( struct STRXXSBASE *, const char *, unsigned int );

struct STRXXSBASE _TRXXSBaseTest( unsigned int );             

                                      /* command processors */
int TRXCmdXS(
              const char *, const char *, const char *,
              const char *, const char *, const char *, 
              int32_t *, int32_t *, int32_t *, int32_t *, int32_t *, int32_t *,
              int32_t *, int32_t *, int32_t *, int32_t *, int32_t *,
              int32_t *, int32_t *, int32_t *,
              const char *, const char *, const char *, const char *, 
              const char *, 
              double *,
              int32_t * 
            );
            
int TRXCmdXSTest      ( const char *, int32_t * );

/*
   --- error handling
*/

int TRXXSError( struct SCMDERR *, unsigned int );

enum { ERRTRXXS_SYS=1, 
       ERRTRXXS_BADADDR,
       ERRTRXXS_BADNUMBER,
       ERRTRXXS_BADEDIM,
       ERRTRXXS_BADPROJ,
       ERRTRXXS_BADTARG,
       ERRTRXXS_BADNSH,
       ERRTRXXS_BADNE,
       ERRTRXXS_BADE,
       ERRTRXXS_BADSIGTOT,
       ERRTRXXS_BADWDIM,
       ERRTRXXS_BADADIM,
       ERRTRXXS_BADSIG2,
       ERRTRXXS_NOSUCHPROJ,
       ERRTRXXS_PROJDUPFAILED,
       ERRTRXXS_TARGDUPFAILED,
       ERRTRXXS_NOSUCHMETHOD,
       ERRTRXXS_BADW,
       ERRTRXXS_BADA,
       ERRTRXXS_BADINTERACTION,
       ERRTRXXS_BADDIMENSION,
       ERRTRXXS_BADXSPROJ,
       ERRTRXXS_BADXSTARG,
       ERRTRXXS_BADITEMLIST,
       ERRTRXXS_NOSUCHMATERIAL,
       ERRTRXXS_EOUTOFRANGE,
       ERRTRXXS_NOTCOMPTON,
       ERRTRXXS_BADCUMW,
       ERRTRXXS_BADCUMA,       
       ERRTRXXS_TARGMATCHFAILED,
       ERRTRXXS_PROJMATCHFAILED,
       ERRTRXXS_NOVALIDXS,
       ERRTRXXS_BASEEMPTY,
       ERRTRXXS_BASENOINIT,
       ERRTRXXS_BADLIST,
       ERRTRXXS_BADFILELIST,
       ERRTRXXS_RUDDUNSUPPTARG,       
       ERRTRXXS_READUNKNOWNTAG,
       ERRTRXXS_READSYNTAX,
       ERRTRXXS_READNODATA,
       ERRTRXXS_READTOOFEWDATA,
       ERRTRXXS_READBADFILETYPE,
       ERRTRXXS_READBADFILEVERSION,
       ERRTRXXS_READBADNUMBER,
       ERRTRXXS_READBADTARGET,
       ERRTRXXS_READBADPROJECTILE,
       ERRTRXXS_READBADSHELL,
       ERRTRXXS_READBADNSHELL,
       ERRTRXXS_READBADE,
       ERRTRXXS_READBADW,
       ERRTRXXS_READBADXSDATA,
       ERRTRXXS_READDUPSHELLNO,
       ERRTRXXS_READDUPE,
       ERRTRXXS_READDUPW,
       ERRTRXXS_BADSDCS,
       ERRTRXXS_BADSDCSVAL,
       ERRTRXXS_BADSDCSCUM,
       ERRTRXXS_BADDDCS,
       ERRTRXXS_BADDDCSVAL,
       ERRTRXXS_BADDDCSCUM,
       ERRTRXXS_TARGCOMPLETEFAILED,
       ERRTRXXS_BADSHELLNO,
       ERRTRXXS_TARGCALCFAILED,
       ERRTRXXS_BADTARGCONSTITUENTS,
       ERRTRXXS_PROJCOPYFAILED,
       ERRTRXXS_TARGCOPYFAILED,
   ERRTRXXS_ISMIXTURE,
   ERRTRXXS_NOSUCHTARGXS,
   ERRTRXXS_GSUNSUPPTARG,
};

#endif
